require 'rails_helper'

describe User do

  it { should be_timestamped_document.with(:created) }
  it { should be_timestamped_document.with(:updated) }
  it { should have_field(:reset_password_token).of_type(String) }
  it { should have_field(:reset_password_sent_at).of_type(Time) }
  it { should have_field(:remember_created_at).of_type(Time) }
  it { should have_field(:sign_in_count).of_type(Integer)
    .with_default_value_of(0) }
  it { should have_field(:current_sign_in_at).of_type(Time) }
  it { should have_field(:last_sign_in_at).of_type(Time) }
  it { should have_field(:current_sign_in_ip).of_type(String) }
  it { should have_field(:last_sign_in_ip).of_type(String) }
  it { should have_field(:email).of_type(String) }
  it { should have_field(:name).of_type(String) }
  it { should have_field(:user_roles).of_type(Array) }
  it { should have_field(:encrypted_password).of_type(String) }
  it { should have_field(:updated).of_type(Mongoid::Boolean) }
  it { should have_field(:batched).of_type(Mongoid::Boolean) }

  it { should have_many(:wines) }
  it { should embed_many(:wine_notifications) }

  it { should validate_presence_of(:email) }
  it { should validate_presence_of(:encrypted_password) }

  it { should validate_uniqueness_of(:email) }

  # it { should have_index_for(email: 1)
  #          .with_options(unique: true) }

  describe 'methods' do

    let!(:admin)            { create(:admin) }
    let!(:interpro)         { build_stubbed(:interpro_with_producers) }
    let!(:producer)         { interpro.producers.first }
    let!(:wine) { build_stubbed(:complete_wine,
                                interpro_id: interpro.id,
                                producer: producer)}
    let!(:winewoo_user) {create(:winewoo_user)}

    it 'should respond to role? with valid value' do
      expect(admin.admin?).to be true
      expect(admin.interpro?).to be false
      expect(admin.producer?).to be false
      expect(interpro.admin?).to be false
      expect(interpro.interpro?).to be true
      expect(interpro.producer?).to be false
      expect(producer.admin?).to be false
      expect(producer.interpro?).to be false
      expect(producer.producer?).to be true
    end

    it 'should respond to updated? with valid value' do
      expect(producer.updated?).to be false
    end

    it 'should respond to batched? with valid value' do
      expect(producer.batched?).to be false
    end

    it 'should have the correct roles' do
      expect(interpro.interpro?).to eq true
      expect(interpro.admin?).to eq false
      expect(interpro.winewoo_user?).to eq false
      expect(interpro.producer?).to eq false
      interpro.user_roles << :admin
      interpro.user_roles << :winewoo_user
      expect(interpro.interpro?).to eq true
      expect(interpro.admin?).to eq true
      expect(interpro.winewoo_user?).to eq true
      expect(interpro.producer?).to eq false
    end
    # Interpro
    it 'should respond to has_ecommerce? with valid value' do
      expect(interpro.has_ecommerce?).to be false
    end
    ###################
    # WinewooUser
    describe '.save_for_interpro' do
      it 'should save the interpro for the winewoo_user' do
        winewoo_user.save_for_interpro(interpro)
        saved_winewoo_user = User.find(winewoo_user.id)
        expect(saved_winewoo_user.viewed_interpro_ids).to eq [interpro.id]
      end
    end

    describe '.save_for_producer' do
      it 'should save the producer the winewoo_user' do
        winewoo_user.save_for_producer(producer)
        saved_winewoo_user = User.find(winewoo_user.id)
        expect(saved_winewoo_user.viewed_producer_ids).to eq [producer.id]
      end
    end
    ###################
  end


  # Admin
  it { should be_timestamped_document.with(:created) }
  it { should be_timestamped_document.with(:updated) }
  it { should have_field(:reset_password_token).of_type(String) }
  it { should have_field(:reset_password_sent_at).of_type(Time) }
  it { should have_field(:remember_created_at).of_type(Time) }
  it { should have_field(:sign_in_count).of_type(Integer)
    .with_default_value_of(0) }
  it { should have_field(:current_sign_in_at).of_type(Time) }
  it { should have_field(:last_sign_in_at).of_type(Time) }
  it { should have_field(:current_sign_in_ip).of_type(String) }
  it { should have_field(:last_sign_in_ip).of_type(String) }
  it { should have_field(:email).of_type(String) }
  it { should have_field(:name).of_type(String) }
  it { should have_field(:encrypted_password).of_type(String) }
  it { should have_field(:updated).of_type(Mongoid::Boolean) }
  it { should have_field(:batched).of_type(Mongoid::Boolean) }

  it { should have_many(:wines) }
  it { should embed_many(:wine_notifications) }

  it { should validate_presence_of(:email) }
  it { should validate_presence_of(:encrypted_password) }

  it { should validate_uniqueness_of(:email) }
  ###################

  # Producer
  it { should be_timestamped_document.with(:created) }
  it { should be_timestamped_document.with(:updated) }
  it { should have_field(:reset_password_token).of_type(String) }
  it { should have_field(:reset_password_sent_at).of_type(Time) }
  it { should have_field(:remember_created_at).of_type(Time) }
  it { should have_field(:sign_in_count).of_type(Integer)
    .with_default_value_of(0) }
  it { should have_field(:current_sign_in_at).of_type(Time) }
  it { should have_field(:last_sign_in_at).of_type(Time) }
  it { should have_field(:current_sign_in_ip).of_type(String) }
  it { should have_field(:last_sign_in_ip).of_type(String) }
  it { should have_field(:email).of_type(String) }
  it { should have_field(:name).of_type(String) }
  it { should have_field(:encrypted_password).of_type(String) }
  it { should have_field(:discount).of_type(Integer)
    .with_default_value_of(0) }
  it { should have_field(:updated).of_type(Mongoid::Boolean) }
  it { should have_field(:batched).of_type(Mongoid::Boolean) }

  it { should have_many(:wines) }
  it { should embed_many(:pictures) }
  it { should embed_many(:estates) }
  it { should embed_many(:wine_notifications) }
  it { should embed_one(:contact) }
  it { should have_and_belong_to_many(:interpros)
    .of_type(User)
    .as_inverse_of(:producers) }

  it { should validate_presence_of(:email) }
  it { should validate_presence_of(:encrypted_password) }

  it { should validate_uniqueness_of(:email) }
  ###################

  # Interpro
  it { should be_timestamped_document.with(:created) }
  it { should be_timestamped_document.with(:updated) }
  it { should have_field(:reset_password_token).of_type(String) }
  it { should have_field(:reset_password_sent_at).of_type(Time) }
  it { should have_field(:remember_created_at).of_type(Time) }
  it { should have_field(:sign_in_count).of_type(Integer)
    .with_default_value_of(0) }
  it { should have_field(:current_sign_in_at).of_type(Time) }
  it { should have_field(:last_sign_in_at).of_type(Time) }
  it { should have_field(:current_sign_in_ip).of_type(String) }
  it { should have_field(:last_sign_in_ip).of_type(String) }
  it { should have_field(:spree_api_key).of_type(String) }
  it { should have_field(:spree_api_url).of_type(String)
    .with_default_value_of("http://localhost:3001") }
  it { should have_field(:email).of_type(String) }
  it { should have_field(:name).of_type(String) }
  it { should have_field(:encrypted_password).of_type(String) }
  it { should have_field(:updated).of_type(Mongoid::Boolean) }
  it { should have_field(:batched).of_type(Mongoid::Boolean) }

  it { should have_many(:wines) }
  it { should embed_many(:pictures) }
  it { should embed_many(:wine_notifications) }
  it { should embed_one(:wine_data_element) }
  it { should have_and_belong_to_many(:producers)
    .of_type(User)
    .as_inverse_of(:interpros) }

  it { should validate_presence_of(:email) }
  it { should validate_presence_of(:encrypted_password) }

  it { should validate_uniqueness_of(:email) }

  # it { should have_index_for(email: 1)
  #          .with_options(unique: true) }
  ###################


  # WinewooUser
  it { should be_timestamped_document.with(:created) }
  it { should be_timestamped_document.with(:updated) }

  it { should have_field(:reset_password_token).of_type(String) }
  it { should have_field(:reset_password_sent_at).of_type(Time) }
  it { should have_field(:remember_created_at).of_type(Time) }
  it { should have_field(:sign_in_count).of_type(Integer)
    .with_default_value_of(0) }
  it { should have_field(:current_sign_in_at).of_type(Time) }
  it { should have_field(:last_sign_in_at).of_type(Time) }
  it { should have_field(:current_sign_in_ip).of_type(String) }
  it { should have_field(:last_sign_in_ip).of_type(String) }
  it { should have_field(:email).of_type(String) }
  it { should have_field(:name).of_type(String) }
  it { should have_field(:encrypted_password).of_type(String) }
  it { should have_field(:updated).of_type(Mongoid::Boolean) }
  it { should have_field(:batched).of_type(Mongoid::Boolean) }

  it { should have_many(:wines) }
  it { should embed_many(:wine_notifications) }

  it { should validate_presence_of(:email) }
  it { should validate_presence_of(:encrypted_password) }

  it { should validate_uniqueness_of(:email) }

  # it { should have_index_for(email: 1)
  #          .with_options(unique: true) }

  it { should have_field(:user_country).of_type(String) }
  it { should have_field(:user_city).of_type(String) }
  it { should have_field(:user_zip).of_type(String) }
  it { should have_field(:interpro_ids).of_type(Array) }
  it { should have_field(:producer_ids).of_type(Array) }
  it { should have_field(:status).of_type(Symbol) }

  it { should have_field(:facebook_id).of_type(String) }
  it { should have_field(:twitter_id).of_type(String) }
  it { should have_field(:google_id).of_type(String) }
  it { should have_field(:linkedin_id).of_type(String) }

  it { should have_field(:reputation).of_type(Integer) }

  # it { should have_index_for(interpro_ids: 1) }
  # it { should have_index_for(producer_ids: 1) }

  it { should embed_many(:history_entries) }
  it { should embed_many(:cellar_entries) }
  it { should embed_many(:favorites) }
  it { should have_many(:user_comments) }
  it { should embed_many(:winewoo_user_devices) }

  it { should have_and_belong_to_many(:up_comments).as_inverse_of(:up_votes) }
  it { should have_and_belong_to_many(:down_comments).as_inverse_of(:down_votes) }
  ###################
end

